#!/usr/bin/env python3

"""
Copyright 2024 Panasonic Advanced Technology Development Co.,Ltd. (Liu Yang)
Distributed under MIT license. See LICENSE for more information.
"""

from sensor_msgs.msg import PointCloud2
import rospy
import numpy as np
import argparse
import q3dviewer as q3d
from q3dviewer.Qt.QtWidgets import QLabel, QLineEdit, QDoubleSpinBox, QSpinBox, QWidget, QVBoxLayout, QHBoxLayout, QPushButton
from q3dviewer.Qt import QtCore
from q3dviewer.utils.convert_ros_msg import convert_pointcloud2_msg
from q3dviewer.utils.maths import matrix_to_quaternion, euler_to_matrix, matrix_to_euler


try:
    import open3d as o3d
except ImportError:
    print("\033[91mWarning: open3d is not installed. Please install it to use this tool.\033[0m")
    print("\033[93mYou can install it using: pip install open3d\033[0m")
    exit(1)


viewer = None
cloud0_accum = None
clouds0 = []
cloud1_accum = None
clouds1 = []


class CustomDoubleSpinBox(QDoubleSpinBox):
    def __init__(self, decimals=4, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.decimals = decimals
        self.setDecimals(self.decimals)

    def textFromValue(self, value):
        return f"{value:.{self.decimals}f}"

    def valueFromText(self, text):
        return float(text)


class LiDARCalibViewer(q3d.Viewer):
    def __init__(self, **kwargs):
        self.t01 = np.array([0, 0, 0])
        self.R01 = np.eye(3)
        self.cloud_num = 10
        self.radius = 0.2
        super().__init__(**kwargs)

    def default_gl_setting(self, glwidget):
        # Set camera position and background color
        glwidget.set_bg_color('#ffffff')
        glwidget.set_cam_position(distance=5)
    
    def add_control_panel(self, main_layout):
        # Create a vertical layout for the settings
        setting_layout = QVBoxLayout()
        setting_layout.setAlignment(QtCore.Qt.AlignTop)
        # Add XYZ spin boxes
        label_xyz = QLabel("Set XYZ:")
        setting_layout.addWidget(label_xyz)
        self.box_x = CustomDoubleSpinBox()
        self.box_x.setSingleStep(0.01)
        setting_layout.addWidget(self.box_x)
        self.box_y = CustomDoubleSpinBox()
        self.box_y.setSingleStep(0.01)
        setting_layout.addWidget(self.box_y)
        self.box_z = CustomDoubleSpinBox()
        self.box_z.setSingleStep(0.01)
        setting_layout.addWidget(self.box_z)
        self.box_x.setRange(-100.0, 100.0)
        self.box_y.setRange(-100.0, 100.0)
        self.box_z.setRange(-100.0, 100.0)

        # Add RPY spin boxes
        label_rpy = QLabel("Set Roll-Pitch-Yaw:")
        setting_layout.addWidget(label_rpy)
        self.box_roll = CustomDoubleSpinBox()
        self.box_roll.setSingleStep(0.01)
        setting_layout.addWidget(self.box_roll)
        self.box_pitch = CustomDoubleSpinBox()
        self.box_pitch.setSingleStep(0.01)
        setting_layout.addWidget(self.box_pitch)
        self.box_yaw = CustomDoubleSpinBox()
        self.box_yaw.setSingleStep(0.01)
        setting_layout.addWidget(self.box_yaw)
        self.box_roll.setRange(-np.pi, np.pi)
        self.box_pitch.setRange(-np.pi, np.pi)
        self.box_yaw.setRange(-np.pi, np.pi)

        label_cloud_num = QLabel("Set cloud frame number:")
        setting_layout.addWidget(label_cloud_num)
        self.box_cloud_num = QSpinBox()
        self.box_cloud_num.setValue(self.cloud_num)
        self.box_cloud_num.setRange(1, 100)
        self.box_cloud_num.valueChanged.connect(self.update_cloud_num)
        setting_layout.addWidget(self.box_cloud_num)

        label_trans_quat = QLabel("The lidar0-lidar1 trans and quat:")
        setting_layout.addWidget(label_trans_quat)
        self.line_trans = QLineEdit()
        self.line_trans.setReadOnly(True)
        setting_layout.addWidget(self.line_trans)
        self.line_quat = QLineEdit()
        self.line_quat.setReadOnly(True)
        setting_layout.addWidget(self.line_quat)

        label_matching_setting = QLabel("Set matching radius:")
        setting_layout.addWidget(label_matching_setting)
        self.box_radius = CustomDoubleSpinBox(decimals=2)
        self.box_radius.setSingleStep(0.1)
        self.box_radius.setRange(0.1, 3.0)
        self.box_radius.setValue(self.radius)
        setting_layout.addWidget(self.box_radius)

        self.icp_button = QPushButton("Auto Scan Matching")
        self.icp_button.clicked.connect(self.perform_matching)
        setting_layout.addWidget(self.icp_button)

        self.line_trans.setText(
            f"[{self.t01[0]:.6f}, {self.t01[1]:.6f}, {self.t01[2]:.6f}]")
        quat = matrix_to_quaternion(self.R01)
        self.line_quat.setText(
            f"[{quat[0]:.6f}, {quat[1]:.6f}, {quat[2]:.6f}, {quat[3]:.6f}]")

        # Connect spin boxes to methods
        self.box_x.valueChanged.connect(self.update_xyz)
        self.box_y.valueChanged.connect(self.update_xyz)
        self.box_z.valueChanged.connect(self.update_xyz)
        self.box_roll.valueChanged.connect(self.update_rpy)
        self.box_pitch.valueChanged.connect(self.update_rpy)
        self.box_yaw.valueChanged.connect(self.update_rpy)

        main_layout.addLayout(setting_layout)
        

    def update_radius(self):
        self.radius = self.box_radius.value()

    def update_cloud_num(self):
        self.cloud_num = self.box_cloud_num.value()

    def update_xyz(self):
        x = self.box_x.value()
        y = self.box_y.value()
        z = self.box_z.value()
        self.t01 = np.array([x, y, z])
        self.line_trans.setText(f"[{x:.6f}, {y:.6f}, {z:.6f}]")

    def update_rpy(self):
        roll = self.box_roll.value()
        pitch = self.box_pitch.value()
        yaw = self.box_yaw.value()
        self.R01 = euler_to_matrix(np.array([roll, pitch, yaw]))
        quat = matrix_to_quaternion(self.R01)
        self.line_quat.setText(
            f"[{quat[0]:.6f}, {quat[1]:.6f}, {quat[2]:.6f}, {quat[3]:.6f}]")

    def perform_matching(self):
        global cloud0_accum, cloud1_accum

        if o3d is None:
            print("Error: open3d is not available. Cannot perform matching.")
            return

        if cloud0_accum is not None and cloud1_accum is not None:
            # Convert to Open3D point clouds
            cloud0_o3d = o3d.geometry.PointCloud()
            cloud0_o3d.points = o3d.utility.Vector3dVector(cloud0_accum['xyz'])
            cloud1_o3d = o3d.geometry.PointCloud()
            cloud1_o3d.points = o3d.utility.Vector3dVector(cloud1_accum['xyz'])
            voxel_size = 0.1
            cloud0_o3d = cloud0_o3d.voxel_down_sample(voxel_size)
            cloud1_o3d = cloud1_o3d.voxel_down_sample(voxel_size)
            cloud0_o3d.estimate_normals(
                search_param=o3d.geometry.KDTreeSearchParamHybrid
                (radius=self.radius, max_nn=30))
            trans_init = np.eye(4)
            trans_init[:3, :3] = self.R01
            trans_init[:3, 3] = self.t01

            criteria = o3d.pipelines.registration.ICPConvergenceCriteria(
                relative_fitness=1e-4,
                max_iteration=50)

            # Auto Scan Matching
            reg_p2p = o3d.pipelines.registration.registration_icp(
                cloud1_o3d, cloud0_o3d, self.radius, trans_init,
                o3d.pipelines.registration.
                TransformationEstimationPointToPlane(),
                criteria)

            transformation_icp = reg_p2p.transformation
            # print("ICP Transformation:", transformation_icp)

            # Update R01 and t01 with ICP result
            R01 = transformation_icp[:3, :3]
            t01 = transformation_icp[:3, 3]

            # Update the UI with new values
            quat = matrix_to_quaternion(R01)
            rpy = matrix_to_euler(R01)
            self.box_roll.setValue(rpy[0])
            self.box_pitch.setValue(rpy[1])
            self.box_yaw.setValue(rpy[2])
            self.box_x.setValue(t01[0])
            self.box_y.setValue(t01[1])
            self.box_z.setValue(t01[2])
            self.line_trans.setText(
                f"[{t01[0]:.6f}, {t01[1]:.6f}, {t01[2]:.6f}]")
            self.line_quat.setText(
                f"[{quat[0]: .6f}, {quat[1]: .6f}, {quat[2]: .6f}, {quat[3]: .6f}]")
            self.t01 = t01
            self.R01 = R01
            print("Matching results")
            print(f"translation: [{t01[0]:.6f}, {t01[1]:.6f}, {t01[2]:.6f}]")
            print(
                f"Roll-Pitch-Yaw: [{rpy[0]:.6f}, {rpy[1]:.6f}, {rpy[2]:.6f}]")
            print(
                f"Quaternion: [{quat[0]: .6f}, {quat[1]: .6f}, {quat[2]: .6f}, {quat[3]: .6f}]")


def scan0_cb(data):
    global viewer, clouds0, cloud0_accum
    cloud, _, _  = convert_pointcloud2_msg(data)
    while len(clouds0) > viewer.cloud_num:
        clouds0.pop(0)
    clouds0.append(cloud)
    cloud0_accum = np.concatenate(clouds0)
    viewer['scan0'].set_data(data=cloud0_accum)


def scan1_cb(data):
    global viewer, clouds1, cloud1_accum
    cloud, _, _  = convert_pointcloud2_msg(data)
    while len(clouds1) > viewer.cloud_num:
        clouds1.pop(0)
    clouds1.append(cloud)
    cloud1_accum = np.concatenate(clouds1)
    cloud0_accum_new = cloud1_accum.copy()
    cloud0_accum_new['xyz'] = (
        viewer.R01 @ cloud1_accum['xyz'].T + viewer.t01[:, np.newaxis]).T

    viewer['scan1'].set_data(data=cloud0_accum_new)


def main():
    global viewer
    # Set up argument parser
    parser = argparse.ArgumentParser(
        description="Configure topic names for LiDAR, image, and camera info.")
    parser.add_argument('--lidar0', type=str, default='/lidar0',
                        help="Topic name for LiDAR data")
    parser.add_argument('--lidar1', type=str, default='/lidar1',
                        help="Topic name for camera image data")
    args = parser.parse_args()

    app = q3d.QApplication(["LiDAR Calib"])
    viewer = LiDARCalibViewer(name='LiDAR Calib')
    grid_item = q3d.GridItem(size=10, spacing=1, color='#00000040')
    scan0_item = q3d.CloudItem(
        size=2, alpha=1, color_mode='FLAT', color='#ff0000')
    scan1_item = q3d.CloudItem(
        size=2, alpha=1, color_mode='FLAT', color='#00ff00')
    viewer.add_items(
        {'scan0': scan0_item, 'scan1': scan1_item, 'grid': grid_item})

    rospy.init_node('lidar_calib', anonymous=True)

    # Use topic names from arguments
    rospy.Subscriber(args.lidar0, PointCloud2, scan0_cb, queue_size=1)
    rospy.Subscriber(args.lidar1, PointCloud2, scan1_cb, queue_size=1)

    viewer.show()
    app.exec()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""
Copyright 2024 Panasonic Advanced Technology Development Co.,Ltd. (Liu Yang)
Distributed under MIT license. See LICENSE for more information.
"""

import numpy as np
import q3dviewer as q3d
from q3dviewer.Qt.QtWidgets import QVBoxLayout, QDialog, QLabel
from q3dviewer.Qt.QtCore import QThread, Signal, Qt
from q3dviewer import GLWidget

class ProgressWindow(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Loading")
        self.setModal(True)
        self.setMinimumWidth(400)
        self.label = QLabel(self)
        self.label.setAlignment(Qt.AlignCenter)
        layout = QVBoxLayout()
        layout.addWidget(self.label)
        self.setLayout(layout)

    def update_progress(self, current, total, file_name):
        text = f"[{current}/{total}] loading file: {file_name}"
        self.label.setText(text)


class FileLoaderThread(QThread):
    progress = Signal(int, int, str)  # current, total, filename
    finished = Signal()

    def __init__(self, viewer, files):
        super().__init__()
        self.viewer = viewer
        self.files = files

    def run(self):
        cloud_item = self.viewer['cloud']
        mesh_item = self.viewer['mesh']
        total = len(self.files)
        for i, url in enumerate(self.files):
            # if the file is a mesh file, use mesh_item to load
            file_path = url.toLocalFile()
            import os
            file_name = os.path.basename(file_path)
            self.progress.emit(i + 1, total, file_name)
            
            if url.toLocalFile().lower().endswith(('.stl')):
                from q3dviewer.utils.cloud_io import load_stl
                mesh = load_stl(file_path)
                mesh_item.set_data(mesh)
            else:
                cloud = cloud_item.load(file_path, append=(i > 0))
                center = np.nanmean(cloud['xyz'].astype(np.float64), axis=0)
                self.viewer.glwidget.set_cam_position(center=center)
        self.finished.emit()


class CustomGLWidget(GLWidget):
    def __init__(self, viewer):
        super().__init__()
        self.viewer = viewer
        self.selected_points = []

    def mousePressEvent(self, event):
        # ctrl + left click to add select point
        # ctrl + right click to remove a point from selected points
        if event.button() == Qt.LeftButton and event.modifiers() & Qt.ControlModifier:
            x, y = event.x(), event.y()
            p = self.get_point(x, y)
            if p is not None:
                self.selected_points.append(p)
                self.update_marker()
        elif event.button() == Qt.RightButton and event.modifiers() & Qt.ControlModifier:
            if len(self.selected_points) > 0:
                self.selected_points.pop()
                self.update_marker()
        super().mouseReleaseEvent(event)

    def update_marker(self):
        marks = []
        for p in self.selected_points:
            m = {'text': '',
                 'position': p,
                 'color': (0.0, 1.0, 0.0, 1.0),
                 'font_size': 16,
                 'point_size': 5.0,
                 'line_width': 1.0}
            marks.append(m)
        
        # calculate distances between consecutive points
        self.viewer['marker'].set_data(data=marks, append=False)
        if len(self.selected_points) >= 2:
            total_dists = 0.0
            for i in range(1, len(self.selected_points)):
                p1 = self.selected_points[i - 1]
                p2 = self.selected_points[i]
                dist = np.linalg.norm(np.array(p2) - np.array(p1))
                total_dists += dist
            self.viewer['text'].set_data(text=f'Total Distance: {total_dists:.2f} meter')
        else:
            self.viewer['text'].set_data(text='')


class CloudViewer(q3d.Viewer):
    def __init__(self, **kwargs):
        gl_widget_class=lambda: CustomGLWidget(self)
        super(CloudViewer, self).__init__(
            **kwargs,  gl_widget_class=gl_widget_class)
        self.setAcceptDrops(True)

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):
        """
        Overwrite the drop event to open the cloud file.
        """
        self.progress_window = ProgressWindow(self)
        self.progress_window.show()
        files = event.mimeData().urls()
        self.progress_thread = FileLoaderThread(self, files)
        self.progress_thread.progress.connect(self.file_loading_progress)
        self.progress_thread.finished.connect(self.file_loading_finished)
        self.progress_thread.start()

    def file_loading_progress(self, current, total, file_name):
        self.progress_window.update_progress(current, total, file_name)

    def file_loading_finished(self):
        self.progress_window.close()

    def open_cloud_file(self, file, append=False):
        cloud_item = self['cloud']
        if cloud_item is None:
            print("Can't find clouditem.")
            return
        cloud = cloud_item.load(file, append=append)
        center = np.nanmean(cloud['xyz'].astype(np.float64), axis=0)
        self.glwidget.set_cam_position(center=center)

# print a quick help message using rich
def print_help():
    from rich.console import Console
    from rich.panel import Panel
    from rich.table import Table
    from rich.text import Text
    
    console = Console()
    
    # Create a table for better organization
    table = Table(show_header=False, box=None, padding=(0, 2))
    table.add_column(style="bold cyan", width=30)
    table.add_column(style="white")
    
    # File loading section
    table.add_row("📁 Load Files","Drag and drop files into the viewer")
    table.add_row("","[dim]• Point clouds: .pcd, .ply, .las, .e57[/dim]")
    table.add_row("","[dim]• Mesh files: .stl[/dim]")
    table.add_row("", "")
    
    # Measurement section
    table.add_row("📏 Measure Distance", "Interactive point measurement")
    table.add_row("","[dim]• Ctrl + Left Click: Add measurement point[/dim]")
    table.add_row("","[dim]• Ctrl + Right Click: Remove last point[/dim]")
    table.add_row("","[dim]• Total distance displayed automatically[/dim]")
    table.add_row("", "")
    
    # Camera controls
    table.add_row("🎥 Camera Controls","Navigate the 3D scene")
    table.add_row("","[dim]• Double Click: Set camera center to point[/dim]")
    table.add_row("","[dim]• Right Drag: Rotate view[/dim]")
    table.add_row("","[dim]• Left Drag: Pan view[/dim]")
    table.add_row("","[dim]• Mouse Wheel: Zoom in/out[/dim]")
    table.add_row("", "")
    
    # Settings section
    table.add_row("⚙️  Settings","Press [bold green]'M'[/bold green] to open settings window")
    table.add_row("","[dim]Adjust visualization properties[/dim]")
    
    # Print title and table without border
    console.print()
    console.print("[bold magenta]☁️  Cloud Viewer Help[/bold magenta]\n")
    console.print(table)
    console.print()

def main():
    print_help()
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("--path", help="the cloud file path")
    args = parser.parse_args()
    app = q3d.QApplication(['Cloud Viewer'])
    viewer = CloudViewer(name='Cloud Viewer')
    cloud_item = q3d.CloudIOItem(size=1, alpha=0.1)
    axis_item = q3d.AxisItem(size=0.5, width=5)
    axis_item.disable_setting()
    grid_item = q3d.GridItem(size=1000, spacing=20)
    marker_item = q3d.Text3DItem()  # Changed from CloudItem to Text3DItem
    text_item = q3d.Text2DItem(pos=(20, 40), text="", color='lime', size=16)
    text_item.disable_setting()
    mesh_item = q3d.MeshItem()  # Added MeshIOItem for mesh support

    viewer.add_items(
        {'marker': marker_item, 
         'cloud': cloud_item,
         'mesh': mesh_item,
         'grid': grid_item, 
         'axis': axis_item, 
         'text': text_item,})

    if args.path:
        pcd_fn = args.path
        viewer.open_cloud_file(pcd_fn)

    viewer.show()
    app.exec()


if __name__ == '__main__':
    main()



"""
Copyright 2024 Panasonic Advanced Technology Development Co.,Ltd. (Liu Yang)
Distributed under MIT license. See LICENSE for more information.
"""

from q3dviewer.Qt.QtCore import Qt, Signal
from q3dviewer.Qt.QtGui import QPainter, QColor
from q3dviewer.Qt.QtWidgets import QSlider, QToolTip


class RangeSlider(QSlider):
    # Signal emitted when the range changes
    rangeChanged = Signal(int, int)

    def __init__(self, orientation=Qt.Horizontal,
                 parent=None, vmin=0, vmax=255):
        super().__init__(orientation, parent)
        self.setMinimum(vmin)
        self.setMaximum(vmax)
        self.lower_value = vmin
        self.upper_value = vmax
        self.active_handle = None
        self.bar_color = QColor(200, 200, 200)  # Gray bar
        self.highlight_color = QColor(100, 100, 255)  # Blue for selected range
        self.handle_color = QColor(50, 50, 255)  # Blue handles

        self.setTickPosition(QSlider.NoTicks)  # Hide original ticks
        # Hide slider handle
        self.setStyleSheet("QSlider::handle { background: transparent; }")

    def mousePressEvent(self, event):
        """Override to handle which handle is selected."""
        pos = self.pixelPosToValue(event.pos())
        if abs(pos - self.lower_value) < abs(pos - self.upper_value):
            self.active_handle = "lower"
        else:
            self.active_handle = "upper"

    def mouseMoveEvent(self, event):
        """Override to update handle positions, always clamp and int for cross-platform safety."""
        if event.buttons() != Qt.LeftButton:
            return

        pos = self.pixelPosToValue(event.pos())
        minv, maxv = self.minimum(), self.maximum()
        if self.active_handle == "lower":
            self.lower_value = max(minv, min(pos, self.upper_value - 1))
            self.lower_value = int(round(self.lower_value))
            QToolTip.showText(event.globalPos(), f"Lower: {self.lower_value:.1f}")
        elif self.active_handle == "upper":
            self.upper_value = min(maxv, max(pos, self.lower_value + 1))
            self.upper_value = int(round(self.upper_value))
            QToolTip.showText(event.globalPos(), f"Upper: {self.upper_value:.1f}")
        # Always emit clamped, int values
        self.rangeChanged.emit(self.lower_value, self.upper_value)
        self.update()

    def mouseReleaseEvent(self, event):
        """Override to hide tooltip when mouse stops."""
        QToolTip.hideText()
        super().mouseReleaseEvent(event)

    def enterEvent(self, event):
        """Show tooltips for lower and upper values when mouse enters."""
        QToolTip.showText(self.mapToGlobal(self.rect().topLeft()), 
                          f"Lower: {self.lower_value:.1f}, Upper: {self.upper_value:.1f}")
        super().enterEvent(event)

    def leaveEvent(self, event):
        """Hide tooltips when mouse leaves."""
        QToolTip.hideText()
        super().leaveEvent(event)

    def paintEvent(self, event):
        """Override to paint custom range handles."""
        painter = QPainter(self)

        # Draw the range bar
        painter.setPen(Qt.NoPen)

        bar_height = 6
        bar_y = self.height() // 2 - bar_height // 2
        painter.setBrush(self.bar_color)
        painter.drawRect(0, bar_y, self.width(), bar_height)

        # Draw the selected range
        lower_x = int(self.valueToPixelPos(self.lower_value))
        upper_x = int(self.valueToPixelPos(self.upper_value))
        painter.setBrush(self.highlight_color)
        painter.drawRect(lower_x, bar_y, upper_x - lower_x, bar_height)

        # Draw the range handles
        painter.setBrush(self.handle_color)
        painter.drawEllipse(lower_x - 5, bar_y - 4, 12, 12)
        painter.drawEllipse(upper_x - 5, bar_y - 4, 12, 12)

        painter.end()

    def pixelPosToValue(self, pos):
        """Convert pixel position to slider value."""
        return self.minimum() + (self.maximum() - self.minimum()) \
            * pos.x() / self.width()

    def valueToPixelPos(self, value):
        """Convert slider value to pixel position."""
        return self.width() * (value - self.minimum()) /\
            (self.maximum() - self.minimum())



#!/usr/bin/env python3

"""
Copyright 2024 Panasonic Advanced Technology Development Co.,Ltd. (Liu Yang)
Distributed under MIT license. See LICENSE for more information.
"""

"""
You can use the following code to set the Q3D_QT_IMPL environment variable
# tested for PyQt5, PySide2 and PySide6
# -------------------------------
in your python script before importing QT:
    import os
    os.environ['Q3D_QT_IMPL'] = 'PySide6'
or in your bash:
    export Q3D_QT_IMPL=PyQt5
# -------------------------------
"""

import importlib.util
import builtins
import sys
import os


Q3D_QT_IMPL = os.environ.get('Q3D_QT_IMPL')
Q3D_DEBUG = os.environ.get('Q3D_DEBUG')

if Q3D_QT_IMPL not in ['PyQt5', 'PySide2', 'PySide6']:
    Q3D_QT_IMPL = None

if Q3D_QT_IMPL is None:
    if importlib.util.find_spec('PyQt5') is not None:
        Q3D_QT_IMPL = 'PyQt5'
    elif importlib.util.find_spec('PySide6') is not None:
        Q3D_QT_IMPL = 'PySide6'
    else:
        raise ImportError('No Qt binding found. Please install PySide6.')


def import_module(name):
    module = builtins.__import__(f'{Q3D_QT_IMPL}.{name}').__dict__[name]
    sys.modules[f'{__name__}.{name}'] = module


def load_qt():
    global Q3D_QT_IMPL, Q3D_DEBUG
    if Q3D_DEBUG is not None:
        print(f'Using {Q3D_QT_IMPL} as Qt binding.')

    # register common Qt modules
    modules = ['QtCore', 'QtGui', 'QtWidgets']
    for name in modules:
        import_module(name)
    # register OpenGL modules for PySide6
    if Q3D_QT_IMPL == 'PySide6':
        import_module('QtOpenGLWidgets')
        sys.modules[f'{__name__}.QtWidgets'].QOpenGLWidget = sys.modules[f'{__name__}.QtOpenGLWidgets'].QOpenGLWidget
    # make PyQt5 and PySide6 modules compatible
    if Q3D_QT_IMPL == 'PyQt5':
        sys.modules[f'{__name__}.QtCore'].Signal = sys.modules[f'{__name__}.QtCore'].pyqtSignal
        sys.modules[f'{__name__}.QtCore'].Slot = sys.modules[f'{__name__}.QtCore'].pyqtSlot
        sys.modules[f'{__name__}.QtCore'].Property = sys.modules[f'{__name__}.QtCore'].pyqtProperty
    # Handle exec() for PySide2
    if Q3D_QT_IMPL == 'PySide2':
        sys.modules[f'{__name__}.QtWidgets'].QApplication.exec = sys.modules[f'{__name__}.QtWidgets'].QApplication.exec_


load_qt()



#!/usr/bin/env python3

import q3dviewer as q3d  # Import q3dviewer

def main():
    # Create a Qt application
    app = q3d.QApplication([])

    # Create various 3D items
    axis_item = q3d.AxisItem(size=0.5, width=5)
    grid_item = q3d.GridItem(size=10, spacing=1)

    # Create a viewer
    viewer = q3d.Viewer(name='example')
    
    # Add items to the viewer
    viewer.add_items({
        'grid': grid_item,
        'axis': axis_item,
    })

    # Show the viewer & run the Qt application
    viewer.show()
    app.exec()

if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""
Copyright 2024 Panasonic Advanced Technology Development Co.,Ltd. (Liu Yang)
Distributed under MIT license. See LICENSE for more information.
"""


import numpy as np
from q3dviewer.base_item import BaseItem
from OpenGL.GL import *
from OpenGL.GL import shaders
from q3dviewer.Qt.QtWidgets import QLabel, QCheckBox, QDoubleSpinBox, QSlider, QHBoxLayout, QLineEdit

import os
from q3dviewer.utils import set_uniform, text_to_rgba
import time



class MeshItem(BaseItem):
    """
    A OpenGL mesh item for rendering 3D triangular meshes.
    Attributes:
        color (str or tuple): Accepts any valid matplotlib color (e.g., 'red', '#FF4500', (1.0, 0.5, 0.0)).
        wireframe (bool): If True, renders the mesh in wireframe mode.
    """
    def __init__(self, color='lightblue', wireframe=False):
        super(MeshItem, self).__init__()
        self.wireframe = wireframe
        self.color = color
        self.flat_rgb = text_to_rgba(color, flat=True)
        
        # Incremental buffer management
        self.FACE_CAPACITY = 1000000    # Initial capacity for faces
        
        # Faces buffer: N x 13 numpy array
        # Each row: [v0.x, v0.y, v0.z, v1.x, v1.y, v1.z, v2.x, v2.y, v2.z, v3.x, v3.y, v3.z, good]
        self.faces = np.zeros((self.FACE_CAPACITY, 13), dtype=np.float32)
        
        # valid_f_top: pointer to end of valid faces
        self.valid_f_top = 0
        
        # key2index: mapping from face_key to face buffer index
        self.key2index = {}  # {face_key: face_index}
        
        # OpenGL objects
        self.vao = None
        self.vbo = None
        self.program = None
        self._gpu_face_capacity = 0    # Track GPU buffer capacity
        
        # Fixed rendering parameters (not adjustable via UI)
        self.enable_lighting = True
        self.line_width = 1.0
        self.light_pos = [1.0, 1.0, 1.0]
        self.light_color = [1.0, 1.0, 1.0]
        self.ambient_strength = 0.1
        self.diffuse_strength = 1.2
        self.specular_strength = 0.1
        self.shininess = 32.0
        self.alpha = 1.0
        
        # Settings flag
        self.need_update_setting = True
        self.path = os.path.dirname(__file__)
    
        
    def add_setting(self, layout):
        """Add UI controls for mesh visualization"""
        # Only keep wireframe toggle - all other parameters are fixed
        self.wireframe_box = QCheckBox("Wireframe Mode")
        self.wireframe_box.setChecked(self.wireframe)
        self.wireframe_box.toggled.connect(self.update_wireframe)
        layout.addWidget(self.wireframe_box)

        # Enable lighting toggle
        self.lighting_box = QCheckBox("Enable Lighting")
        self.lighting_box.setChecked(self.enable_lighting)
        self.lighting_box.toggled.connect(self.update_enable_lighting)
        layout.addWidget(self.lighting_box)

        label_rgb = QLabel("Color:")
        label_rgb.setToolTip("Use hex color, i.e. #FF4500, or named color, i.e. 'red'")
        layout.addWidget(label_rgb)
        self.edit_rgb = QLineEdit()
        self.edit_rgb.setToolTip("Use hex color, i.e. #FF4500, or named color, i.e. 'red'")
        self.edit_rgb.setText(self.color)
        self.edit_rgb.textChanged.connect(self._on_color)
        layout.addWidget(self.edit_rgb)

        # Material property controls for Phong lighting
        if self.enable_lighting:
            # Ambient strength control (slider 0-100 mapped to 0.0-1.0)
            ambient_layout = QHBoxLayout()
            ambient_label = QLabel("Ambient Strength:")
            ambient_layout.addWidget(ambient_label)
            self.ambient_slider = QSlider()
            self.ambient_slider.setOrientation(1)  # Qt.Horizontal
            self.ambient_slider.setRange(0, 100)
            self.ambient_slider.setValue(int(self.ambient_strength * 100))
            self.ambient_slider.valueChanged.connect(lambda v: self.update_ambient_strength(v / 100.0))
            ambient_layout.addWidget(self.ambient_slider)
            layout.addLayout(ambient_layout)

            # Diffuse strength control (slider 0-200 mapped to 0.0-2.0)
            diffuse_layout = QHBoxLayout()
            diffuse_label = QLabel("Diffuse Strength:")
            diffuse_layout.addWidget(diffuse_label)
            self.diffuse_slider = QSlider()
            self.diffuse_slider.setOrientation(1)
            self.diffuse_slider.setRange(0, 200)
            self.diffuse_slider.setValue(int(self.diffuse_strength * 100))
            self.diffuse_slider.valueChanged.connect(lambda v: self.update_diffuse_strength(v / 100.0))
            diffuse_layout.addWidget(self.diffuse_slider)
            layout.addLayout(diffuse_layout)

            # Specular strength control (slider 0-200 mapped to 0.0-2.0)
            specular_layout = QHBoxLayout()
            specular_label = QLabel("Specular Strength:")
            specular_layout.addWidget(specular_label)
            self.specular_slider = QSlider()
            self.specular_slider.setOrientation(1)
            self.specular_slider.setRange(0, 200)
            self.specular_slider.setValue(int(self.specular_strength * 100))
            self.specular_slider.valueChanged.connect(lambda v: self.update_specular_strength(v / 100.0))
            specular_layout.addWidget(self.specular_slider)
            layout.addLayout(specular_layout)

            # Shininess control (slider 1-256 mapped to 1-256)
            shininess_layout = QHBoxLayout()
            shininess_label = QLabel("Shininess:")
            shininess_layout.addWidget(shininess_label)
            self.shininess_slider = QSlider()
            self.shininess_slider.setOrientation(1)
            self.shininess_slider.setRange(1, 256)
            self.shininess_slider.setValue(int(self.shininess))
            self.shininess_slider.valueChanged.connect(lambda v: self.update_shininess(float(v)))
            shininess_layout.addWidget(self.shininess_slider)
            layout.addLayout(shininess_layout)

    def _on_color(self, color):
        try:
            self.color = color
            self.flat_rgb = text_to_rgba(color, flat=True)
            self.need_update_setting = True
        except ValueError:
            pass

    def update_wireframe(self, value):
        self.wireframe = value
        
    def update_enable_lighting(self, value):
        self.enable_lighting = value
        self.need_update_setting = True
        
    def update_line_width(self, value):
        self.line_width = value
        self.need_update_setting = True
        
    def update_ambient_strength(self, value):
        self.ambient_strength = value
        self.need_update_setting = True
        
    def update_diffuse_strength(self, value):
        self.diffuse_strength = value
        self.need_update_setting = True
        
    def update_specular_strength(self, value):
        self.specular_strength = value
        self.need_update_setting = True
        
    def update_shininess(self, value):
        self.shininess = value
        self.need_update_setting = True

    def update_alpha(self, value):
        """Update mesh alpha (opacity)"""
        self.alpha = float(value)
        self.need_update_setting = True

    def set_data(self, data):
        """
        Set complete mesh data at once.
        
        Args:
            data: is Nx3 numpy array (N must be divisible by 3) or dict
            if is dict, uses the dict format: 
            [face_key: (v0.x, v0.y, v0.z, ..., v3.z, good)]
        """
        self.clear_mesh()

        if isinstance(data, dict):
            # Use dict format directly (from get_mesh_data/get_incremental_mesh_data)
            self.set_incremental_data(data)
            return
                
        # Check if Nx3 array

        if not isinstance(data, np.ndarray):
            raise ValueError("Invalid data type")

        good_format = False
        if data.ndim == 2 and \
            data.shape[1] == 3 and \
            data.shape[0] % 3 == 0:
            good_format = True

        if not good_format:
            raise ValueError("Invalid data shape")

        # Convert to Nx13 numpy array
        N = data.shape[0] // 3
        faces = np.zeros((N, 13), dtype=np.float32)
        tmp = data.reshape(N, 9)
        faces[:, 0:3]  = tmp[:, 0:3]   # copy v0
        faces[:, 3:6]  = tmp[:, 3:6]   # copy v1
        faces[:, 6:9]  = tmp[:, 6:9]   # copy v2
        faces[:, 9:12] = tmp[:, 6:9]   # copy v3 from v2 (degenerate quad)
        faces[:, 12] = 1.0             # set good=1.0
        self.faces = faces
        self.valid_f_top = N


    def set_incremental_data(self, fs):
        """
        Incrementally update mesh with new face data.
        Args:
            fs: Dict {face_key: (v0.x, v0.y, v0.z, v1.x, v1.y, v1.z, 
                                  v2.x, v2.y, v2.z, v3.x, v3.y, v3.z, good), ...}
                13-tuple with vertex positions and good flag (0.0 or 1.0)
                If good==1:
                    Triangle 1: (v0, v1, v3)
                    Triangle 2: (v0, v3, v2)
        Updates:
            - faces: updates existing faces or appends new ones
            - key2index: tracks face_key -> face_index mapping
        """
        if not fs:
            return

        # Ensure enough capacity in faces buffer
        # wasted cases are better than frequent expansions
        while self.valid_f_top + len(fs) > len(self.faces):
            self._expand_face_buffer()

        # Optimization: Separate updates from new insertions to avoid
        # dictionary lookup performance degradation during growth
        update_idxs = []   # [idx, ...]
        update_data = []   # [face_data, ...]
        new_keys = []      # [key, ...]
        new_data = []      # [face_data, ...]

        for face_key, face_data in fs.items():
            face_idx = self.key2index.get(face_key)
            if face_idx is not None:
                update_idxs.append(face_idx)
                update_data.append(face_data)
            else:
                new_keys.append(face_key)
                new_data.append(face_data)

        # Batch update existing faces
        if update_data:
            indices = np.array(update_idxs, dtype=np.int32)
            data = np.array(update_data, dtype=np.float32)
            self.faces[indices] = data
        
        # Batch insert new faces
        if new_data:
            n_new = len(new_data)
            data = np.array(new_data, dtype=np.float32)
            self.faces[self.valid_f_top: self.valid_f_top + n_new] = data

            # Update key2index mapping for new faces
            for i, face_key in enumerate(new_keys):
                self.key2index[face_key] = self.valid_f_top + i
            
            self.valid_f_top += n_new
    
    def _expand_face_buffer(self):
        """Expand the faces buffer when capacity is reached"""
        new_capacity = len(self.faces) + self.FACE_CAPACITY
        new_buffer = np.zeros((new_capacity, 13), dtype=np.float32)
        new_buffer[:len(self.faces)] = self.faces
        self.faces = new_buffer
    
    def clear_mesh(self):
        """Clear all mesh data and reset buffers"""
        self.valid_f_top = 0
        self.key2index.clear()
        if hasattr(self, 'indices_array'):
            self.indices_array = np.array([], dtype=np.uint32)

    def initialize_gl(self):
        """OpenGL initialization"""
        # Use instanced mesh shaders with geometry shader for GPU-side triangle generation
        vert_shader = open(self.path + '/../shaders/mesh_vert.glsl', 'r').read()
        geom_shader = open(self.path + '/../shaders/mesh_geom.glsl', 'r').read()
        frag_shader = open(self.path + '/../shaders/mesh_frag.glsl', 'r').read()
        try:
            program = shaders.compileProgram(
                shaders.compileShader(vert_shader, GL_VERTEX_SHADER),
                shaders.compileShader(geom_shader, GL_GEOMETRY_SHADER),
                shaders.compileShader(frag_shader, GL_FRAGMENT_SHADER),
            )
            self.program = program
        except Exception as e:
            raise

    def update_render_buffer(self):
        """
        Update GPU buffer with face data (no separate vertex buffer).
        Each face contains embedded vertex positions (13 floats).
        Geometry shader generates triangles on GPU from face vertices.
        Dynamically resizes GPU buffer when Python buffer expands.
        """
        if self.valid_f_top == 0:
            return
        
        # Initialize buffers on first call
        if self.vao is None:
            self.vao = glGenVertexArrays(1)
            self.vbo = glGenBuffers(1)
            self._gpu_face_capacity = 0
        
        # Check if we need to reallocate VBO for faces
        if self._gpu_face_capacity < len(self.faces):
            glBindVertexArray(self.vao)
            glBindBuffer(GL_ARRAY_BUFFER, self.vbo)
            glBufferData(GL_ARRAY_BUFFER,
                        self.faces.nbytes,
                        None,
                        GL_DYNAMIC_DRAW)
            
            # Setup face attributes (per-instance)
            # Face data: [v0.x, v0.y, v0.z, v1.x, v1.y, v1.z, v2.x, v2.y, v2.z, v3.x, v3.y, v3.z, good]
            # 13 floats = 52 bytes stride
            
            # v0 (location 1) - vec3
            glEnableVertexAttribArray(1)
            glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, 52, ctypes.c_void_p(0))
            glVertexAttribDivisor(1, 1)
            
            # v1 (location 2) - vec3
            glEnableVertexAttribArray(2)
            glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, 52, ctypes.c_void_p(12))
            glVertexAttribDivisor(2, 1)
            
            # v2 (location 3) - vec3
            glEnableVertexAttribArray(3)
            glVertexAttribPointer(3, 3, GL_FLOAT, GL_FALSE, 52, ctypes.c_void_p(24))
            glVertexAttribDivisor(3, 1)
            
            # v3 (location 4) - vec3
            glEnableVertexAttribArray(4)
            glVertexAttribPointer(4, 3, GL_FLOAT, GL_FALSE, 52, ctypes.c_void_p(36))
            glVertexAttribDivisor(4, 1)
            
            # good flag (location 5) - float
            glEnableVertexAttribArray(5)
            glVertexAttribPointer(5, 1, GL_FLOAT, GL_FALSE, 52, ctypes.c_void_p(48))
            glVertexAttribDivisor(5, 1)
            
            glBindVertexArray(0)
            glBindBuffer(GL_ARRAY_BUFFER, 0)
            self._gpu_face_capacity = len(self.faces)
        
        # Upload faces to VBO
        glBindBuffer(GL_ARRAY_BUFFER, self.vbo)
        glBufferSubData(GL_ARRAY_BUFFER,
                       0,
                       self.valid_f_top * 13 * 4,  # 13 floats * 4 bytes per face
                       self.faces[:self.valid_f_top])
        glBindBuffer(GL_ARRAY_BUFFER, 0)
        
    def update_setting(self):
        """Set fixed rendering parameters (called once during initialization)"""
        if not self.need_update_setting:
            return
        # Set fixed uniforms for instanced shaders
        set_uniform(self.program, int(self.enable_lighting), 'if_light')
        set_uniform(self.program, 1, 'two_sided')

        set_uniform(self.program, np.array(self.light_color, dtype=np.float32), 'light_color')
        set_uniform(self.program, float(self.ambient_strength), 'ambient_strength')
        set_uniform(self.program, float(self.diffuse_strength), 'diffuse_strength')
        set_uniform(self.program, float(self.specular_strength), 'specular_strength')
        set_uniform(self.program, float(self.shininess), 'shininess')
        set_uniform(self.program, float(self.alpha), 'alpha')
        set_uniform(self.program, int(self.flat_rgb), 'flat_rgb')
        self.need_update_setting = False

    def paint(self):
        """
        Render the mesh using instanced rendering with geometry shader.
        Each face instance is rendered as a point, geometry shader generates 2 triangles.
        GPU filters faces based on good flag.
        """
        if self.valid_f_top == 0:
            return
        
        glUseProgram(self.program)
    
        self.update_render_buffer()
        self.update_setting()
        
        view_matrix = self.glwidget().view_matrix
        set_uniform(self.program, view_matrix, 'view')
        project_matrix = self.glwidget().projection_matrix
        set_uniform(self.program, project_matrix, 'projection')
        view_pos = self.glwidget().center
        set_uniform(self.program, np.array(view_pos), 'view_pos')
            
        # Enable blending and depth testing
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        glEnable(GL_DEPTH_TEST)
        glDisable(GL_CULL_FACE)  # two-sided rendering
        
        # Set line width
        glLineWidth(self.line_width)
        
        # Bind VAO (vertex positions are now in VBO attributes)
        glBindVertexArray(self.vao)
                
        if self.wireframe:
            glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
        else:
            glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
        
        # Draw using instanced rendering
        # Input: POINTS (one per face instance)
        # Geometry shader generates 2 triangles (6 vertices) per point
        glDrawArraysInstanced(GL_POINTS, 0, 1, self.valid_f_top)
            
        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
        glBindVertexArray(0)
        glDisable(GL_DEPTH_TEST)
        glDisable(GL_BLEND)
        glUseProgram(0)
        
